@@ -44,9 +44,11 @@ DATETIME_DELTA = t.Union[
     exp.DatetimeSub,
     exp.TimeAdd,
     exp.TimeSub,
+    exp.TimestampAdd,
     exp.TimestampSub,
     exp.TsOrDsAdd,
 ]
+DATETIME_ADD = (exp.DateAdd, exp.TimeAdd, exp.DatetimeAdd, exp.TsOrDsAdd, exp.TimestampAdd)
 
 if t.TYPE_CHECKING:
     from sqlglot._typing import B, E, F
@@ -539,6 +541,10 @@ class Dialect(metaclass=_Dialect):
     # STRING type (Snowflake's case) or can be of any type
     TRY_CAST_REQUIRES_STRING: t.Optional[bool] = None
 
+    # Whether the double negation can be applied
+    # Not safe with MySQL and SQLite due to type coercion (may not return boolean)
+    SAFE_TO_ELIMINATE_DOUBLE_NEGATION = True
+
     # --- Autofilled ---
 
     tokenizer_class = Tokenizer
@@ -724,6 +730,7 @@ class Dialect(metaclass=_Dialect):
             exp.Pi,
             exp.Pow,
             exp.Quantile,
+            exp.Radians,
             exp.Round,
             exp.SafeDivide,
             exp.Sqrt,
@@ -764,8 +771,12 @@ class Dialect(metaclass=_Dialect):
             exp.TimeAdd,
             exp.TimeSub,
         },
+        exp.DataType.Type.TIMESTAMPLTZ: {
+            exp.TimestampLtzFromParts,
+        },
         exp.DataType.Type.TIMESTAMPTZ: {
             exp.CurrentTimestampLTZ,
+            exp.TimestampTzFromParts,
         },
         exp.DataType.Type.TIMESTAMP: {
             exp.CurrentTimestamp,
@@ -777,10 +788,17 @@ class Dialect(metaclass=_Dialect):
         },
         exp.DataType.Type.TINYINT: {
             exp.Day,
-            exp.Month,
+            exp.DayOfWeek,
+            exp.DayOfWeekIso,
+            exp.DayOfMonth,
+            exp.DayOfYear,
             exp.Week,
-            exp.Year,
+            exp.WeekOfYear,
+            exp.Month,
             exp.Quarter,
+            exp.Year,
+            exp.YearOfWeek,
+            exp.YearOfWeekIso,
         },
         exp.DataType.Type.VARCHAR: {
             exp.ArrayConcat,
@@ -1681,11 +1699,7 @@ def date_delta_to_binary_interval_op(
     def date_delta_to_binary_interval_op_sql(self: Generator, expression: DATETIME_DELTA) -> str:
         this = expression.this
         unit = unit_to_var(expression)
-        op = (
-            "+"
-            if isinstance(expression, (exp.DateAdd, exp.TimeAdd, exp.DatetimeAdd, exp.TsOrDsAdd))
-            else "-"
-        )
+        op = "+" if isinstance(expression, DATETIME_ADD) else "-"
 
         to_type: t.Optional[exp.DATA_TYPE] = None
         if cast:
@@ -1964,6 +1978,15 @@ def sequence_sql(self: Generator, expression: exp.GenerateSeries | exp.GenerateD
     return self.func("SEQUENCE", start, end, step)
 
 
+def build_like(expr_type: t.Type[E]) -> t.Callable[[t.List], E | exp.Escape]:
+    def _builder(args: t.List) -> E | exp.Escape:
+        like_expr = expr_type(this=seq_get(args, 0), expression=seq_get(args, 1))
+        escape = seq_get(args, 2)
+        return exp.Escape(this=like_expr, expression=escape) if escape else like_expr
+
+    return _builder
+
+
 def build_regexp_extract(expr_type: t.Type[E]) -> t.Callable[[t.List, Dialect], E]:
     def _builder(args: t.List, dialect: Dialect) -> E:
         return expr_type(
@@ -2074,3 +2097,19 @@ def build_replace_with_optional_replacement(args: t.List) -> exp.Replace:
         expression=seq_get(args, 1),
         replacement=seq_get(args, 2) or exp.Literal.string(""),
     )
+
+
+def regexp_replace_global_modifier(expression: exp.RegexpReplace) -> exp.Expression | None:
+    modifiers = expression.args.get("modifiers")
+    single_replace = expression.args.get("single_replace")
+    occurrence = expression.args.get("occurrence")
+
+    if not single_replace and (not occurrence or (occurrence.is_int and occurrence.to_py() == 0)):
+        if not modifiers or modifiers.is_string:
+            # Append 'g' to the modifiers if they are not provided since
+            # the semantics of REGEXP_REPLACE from the input dialect
+            # is to replace all occurrences of the pattern.
+            value = "" if not modifiers else modifiers.name
+            modifiers = exp.Literal.string(value + "g")
+
+    return modifiers
