@@ -7,6 +7,7 @@ class TestSQLite(Validator):
     dialect = "sqlite"
 
     def test_sqlite(self):
+        self.validate_identity("SELECT RANK() OVER (RANGE CURRENT ROW) FROM tbl")
         self.validate_identity("UNHEX(a, b)")
         self.validate_identity("SELECT DATE()")
         self.validate_identity("SELECT DATE('now', 'start of month', '+1 month', '-1 day')")
@@ -122,6 +123,22 @@ class TestSQLite(Validator):
             'CREATE TABLE "foo t" ("foo t id" TEXT NOT NULL PRIMARY KEY)',
         )
         self.validate_identity("REPLACE INTO foo (x, y) VALUES (1, 2)", check_command_warning=True)
+        self.validate_identity(
+            "ATTACH DATABASE 'foo' AS schema_name", "ATTACH 'foo' AS schema_name"
+        )
+        self.validate_identity(
+            "ATTACH DATABASE NOT EXISTS(SELECT 1) AS schema_name",
+            "ATTACH NOT EXISTS(SELECT 1) AS schema_name",
+        )
+        self.validate_identity(
+            "ATTACH DATABASE IIF(NOT EXISTS(SELECT 1), 'foo1', 'foo2') AS schema_name",
+            "ATTACH IIF(NOT EXISTS(SELECT 1), 'foo1', 'foo2') AS schema_name",
+        )
+        self.validate_identity(
+            "ATTACH DATABASE 'foo' || '.foo2' AS schema_name",
+            "ATTACH 'foo' || '.foo2' AS schema_name",
+        )
+        self.validate_identity("DETACH DATABASE schema_name", "DETACH schema_name")
 
     def test_strftime(self):
         self.validate_identity("SELECT STRFTIME('%Y/%m/%d', 'now')")
