@@ -7,7 +7,6 @@ from sqlglot.transforms import (
     eliminate_qualify,
     eliminate_window_clause,
     remove_precision_parameterized_types,
-    unalias_group,
 )
 
 
@@ -17,41 +16,10 @@ class TestTransforms(unittest.TestCase):
     def validate(self, transform, sql, target, dialect=None):
         with self.subTest(f"{dialect} - {sql}"):
             self.assertEqual(
-                parse_one(sql, dialect=dialect).transform(transform).sql(dialect=dialect), target
+                parse_one(sql, dialect=dialect).transform(transform).sql(dialect=dialect),
+                target,
             )
 
-    def test_unalias_group(self):
-        self.validate(
-            unalias_group,
-            "SELECT a, b AS b, c AS c, 4 FROM x GROUP BY a, b, x.c, 4",
-            "SELECT a, b AS b, c AS c, 4 FROM x GROUP BY a, 2, x.c, 4",
-        )
-        self.validate(
-            unalias_group,
-            "SELECT TO_DATE(the_date) AS the_date, CUSTOM_UDF(other_col) AS other_col, last_col AS aliased_last, COUNT(*) AS the_count FROM x GROUP BY TO_DATE(the_date), CUSTOM_UDF(other_col), aliased_last",
-            "SELECT TO_DATE(the_date) AS the_date, CUSTOM_UDF(other_col) AS other_col, last_col AS aliased_last, COUNT(*) AS the_count FROM x GROUP BY TO_DATE(the_date), CUSTOM_UDF(other_col), 3",
-        )
-        self.validate(
-            unalias_group,
-            "SELECT SOME_UDF(TO_DATE(the_date)) AS the_date, COUNT(*) AS the_count FROM x GROUP BY SOME_UDF(TO_DATE(the_date))",
-            "SELECT SOME_UDF(TO_DATE(the_date)) AS the_date, COUNT(*) AS the_count FROM x GROUP BY SOME_UDF(TO_DATE(the_date))",
-        )
-        self.validate(
-            unalias_group,
-            "SELECT SOME_UDF(TO_DATE(the_date)) AS new_date, COUNT(*) AS the_count FROM x GROUP BY new_date",
-            "SELECT SOME_UDF(TO_DATE(the_date)) AS new_date, COUNT(*) AS the_count FROM x GROUP BY 1",
-        )
-        self.validate(
-            unalias_group,
-            "SELECT the_date AS the_date, COUNT(*) AS the_count FROM x GROUP BY the_date",
-            "SELECT the_date AS the_date, COUNT(*) AS the_count FROM x GROUP BY 1",
-        )
-        self.validate(
-            unalias_group,
-            "SELECT a AS a FROM x GROUP BY DATE(a)",
-            "SELECT a AS a FROM x GROUP BY DATE(a)",
-        )
-
     def test_eliminate_distinct_on(self):
         self.validate(
             eliminate_distinct_on,
@@ -274,6 +242,42 @@ class TestTransforms(unittest.TestCase):
                 == "SELECT a.id FROM a LEFT JOIN b ON a.id = b.id AND b.d = const"
             )
 
+            # validate parens
+            self.validate(
+                eliminate_join_marks,
+                "select t1.a, t2.b from t1, t2 where (1 = 1) and (t1.id = t2.id1 (+))",
+                "SELECT t1.a, t2.b FROM t1 LEFT JOIN t2 ON t1.id = t2.id1 WHERE (1 = 1)",
+                dialect,
+            )
+
+            # validate a CASE
+            self.validate(
+                eliminate_join_marks,
+                "select t1.a, t2.b from t1, t2 where t1.id = case when t2.id (+) = 'n/a' then null else t2.id (+) end",
+                "SELECT t1.a, t2.b FROM t1 LEFT JOIN t2 ON t1.id = CASE WHEN t2.id = 'n/a' THEN NULL ELSE t2.id END",
+                dialect,
+            )
+
+            # validate OR
+            self.validate(
+                eliminate_join_marks,
+                "select t1.a, t2.b from t1, t2 where t1.id = t2.id1 (+) or t1.id = t2.id2 (+)",
+                "SELECT t1.a, t2.b FROM t1 LEFT JOIN t2 ON t1.id = t2.id1 OR t1.id = t2.id2",
+                dialect,
+            )
+
+            # validate knockout
+            script = """
+                    SELECT c.customer_name,
+                            (SELECT MAX(o.order_date)
+                            FROM orders o
+                            WHERE o.customer_id(+) = c.customer_id) AS latest_order_date
+                    FROM customers c
+                    """
+            self.assertRaises(
+                AssertionError, eliminate_join_marks, parse_one(script, dialect=dialect)
+            )
+
     def test_eliminate_window_clause(self):
         self.validate(
             eliminate_window_clause,
